import QtQuick 2.0
import Widgets 0.1
import "Math.js" as MathLocal

MouseArea {
    id: draggingArea

    property int orientation: Qt.Vertical
    property bool dragging
    property real dragVelocity: 0
    property real dragValue: (orientation == Qt.Vertical ? (mouseY - __pressedPosition.y)
                                                            : (mouseX - __pressedPosition.x))
    property real lateralPosition: orientation == Qt.Horizontal ? MathLocal.clamp(mouseY, 0, height) : MathLocal.clamp(mouseX, 0, width)
    property variant __pressedPosition: Qt.point(0, 0)
    property variant __dragEvents: []
    property bool clickValidated: true
    property bool zeroVelocityCounts: false


    signal dragStart
    signal dragEnd

    onDragValueChanged: {
        if (dragValue != 0 && pressed) {
            dragging = true
        }
    }

    onDraggingChanged: {
        if (dragging) {
            dragStart()
        }
        else {
            dragEnd()
        }
    }

    function updateSpeed() {
        var drag
        var dragTime, dragValue
        var previousDragTime, previousDragValue
        var totalSpeed = 0

        for (var i=0; i<__dragEvents.length; i++) {
            previousDragTime = dragTime
            previousDragValue = dragValue
            drag = __dragEvents[i]
            dragTime = drag[0]
            if (orientation == Qt.Vertical) {
                dragValue = drag[2]
            } else {
                dragValue = drag[1]
            }
            if (previousDragTime) {
                if (dragTime == previousDragTime) dragTime += 1
                totalSpeed += (previousDragValue - dragValue) / (previousDragTime - dragTime)
            }
        }
        if (zeroVelocityCounts || Math.abs(totalSpeed) > 0.001)
            dragVelocity = totalSpeed / __dragEvents.length * 1000
    }

    function cullOldDragEvents() {
        var numberOfCulledEvents = 0
        var currentTime = new Date().getTime()
        var dragTime

        // cull events older than 50 ms but always keep the latest 2 events
        for (var i=0; i<__dragEvents.length-2; i++) {
            dragTime = __dragEvents[i][0]
            if (currentTime - dragTime <= 50) break
            numberOfCulledEvents += 1
        }

        var tmp = __dragEvents
        tmp.splice(0, numberOfCulledEvents)
        __dragEvents = tmp
    }

    function pushDragEvent(event) {
        var tmp = __dragEvents
        tmp.push([new Date().getTime(), event.x, event.y])
        __dragEvents = tmp
        cullOldDragEvents()
        updateSpeed()
    }

    onPositionChanged: {
        if (dragging) {
            pushDragEvent(mouse)
        }
        if (!draggingArea.containsMouse)
            clickValidated = false
    }

    onPressed: {
        __pressedPosition = Qt.point(mouse.x, mouse.y)
        __dragEvents = []
        pushDragEvent(mouse)
        clickValidated = true
    }

    onReleased: {
        dragging = false
        __pressedPosition = Qt.point(mouse.x, mouse.y)
    }
}
