import QtQuick 2.0
import Widgets 0.1

Flickable {
    id: dashContentPage
    contentWidth: dashImage.width
    contentHeight: dashImage.height
    width: ListView.view ? ListView.view.width : shell.width
    height: ListView.view ? ListView.view.height : shell.height
    //clip: true
    flickableDirection: Flickable.VerticalFlick

    signal applicationSelected(string name)

    ImageMipmap {
        id: dashImage
        resource: shell.tablet ? "graphics/tablet/apps.png" : "graphics/phone/apps.png"
        clip: true
        y: -units.dp(22)
    }

    GridView {
        id: spread
        property int horizontalMargin: units.dp(10/1.5)

        interactive: false
        model: dash.spreadModel
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            topMargin: units.dp(130/1.5)
            leftMargin: units.dp(8)
            rightMargin: units.dp(8)
        }
        height: childrenRect.height

        cellWidth: Math.floor(width/3)
        cellHeight: Math.floor(cellWidth * (dashContentPage.height/dashContentPage.width))

        delegate: Item {
            width: spread.cellWidth
            height: spread.cellHeight

            ImageMipmap {
                id: screenshot
                anchors.centerIn: parent
                resource: shell.tablet ? "" : "graphics/phone/screenshots/" + identifier + ".png"
                width: spread.cellWidth - 2 * spread.horizontalMargin
                height: width * spread.cellHeight/spread.cellWidth
                smooth: true
            }

            Rectangle {
                id: highlight
                anchors.fill: screenshot
                color: "white"
                opacity: 0.2
                visible: mouseArea.pressed
            }

            MouseArea {
                id: mouseArea
                anchors.fill: parent
                onClicked: dashContentPage.ListView.view.applicationSelected(identifier)
            }
        }

    }

    Rectangle {
        width: dashContentPage.width
        height: units.dp(20)
        color: "black"
        opacity: 0.25
        anchors.top: spread.bottom
        anchors.topMargin: units.dp(20/1.5)
    }
}
