import QtQuick 2.0
import Widgets 0.1


Item {
    id: dashBar

    property variant model
    property int currentIndex

    property int lineHeigth: units.dp(6/1.5)
    property int initialHeight: 0
    property int maxHeight: units.dp(50)

    width: units.dp(320)
    height: initialHeight
    Behavior on height {NumberAnimation { duration: 300; easing.type: Easing.OutQuad}}

    state: "initial"

    signal itemSelected(int index)

    function startNavigation() {
        timeout.stop()
        dashBar.state = "navigation"
    }

    function stopNavigation() {
        timeout.restart()
    }

    function finishNavigation() {
        dashBar.state = "initial"
    }



    Rectangle {
        color: "black"
        anchors.fill: parent

        Row {
            id: row
            width: shell.tablet ? dashBar.width/2 : dashBar.width
            height: parent.height
            anchors.horizontalCenter: parent.horizontalCenter
            Repeater {
                id: repeater
                model: dashBarModel
                delegate: Item {
                    width: row.width / dashBarModel.count
                    height: parent.height
                    clip: true
                    ImageMipmap {
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.top: parent.top
                        anchors.topMargin: units.dp(20/1.5)
                        resource: "graphics/dashBar/" + iconName
                        height: dashBar.maxHeight / 2.5
                        width: height * sourceSize.width / sourceSize.height
                        opacity: index == currentIndex ? 1 : 1 //same opacity for now
                        smooth: true
                    }
                    MouseArea {
                        anchors.fill: parent
                        onPressed: {
                            if (dashbar.state == "initial") {
                                dashbar.state = "navigation"
                                mouse.accepted = false
                            }
                        }

                        onClicked: {
                            timeout.restart()
                            dashBar.itemSelected(index)
                        }
                    }
                }
            }
        }
    }

    ListModel {
        id: dashBarModel

        ListElement {
            iconName: "music_lense_icon.png"
        }
        ListElement {
            iconName: "people_lense_icon.png"
        }
        ListElement {
            iconName: "home_lense_icon.png"
        }
        ListElement {
            iconName: "app_lense_icon.png"
        }
        ListElement {
            iconName: "video_lense_icon.png"
        }
    }

    Rectangle {
        id: orangeBar
        color: "#dd4814"
        height: parent.lineHeigth
        width: row.width / model.count
        anchors.bottom: parent.bottom
        x: row.x + currentIndex * width

        Behavior on x {NumberAnimation { duration: 150; easing.type: Easing.OutCubic}}
    }

    Timer {
        id: timeout
        interval: 1500
        running: false
        repeat: false
        onTriggered: finishNavigation()
    }

    states: [
        State {
            name: "initial"
            PropertyChanges {
                target: dashBar
                height: initialHeight
            }
            PropertyChanges {
                target: orangeBar
                height: lineHeigth
            }
        },
        State {
            name: "navigation"
            PropertyChanges {
                target: dashBar
                height: maxHeight
            }
            PropertyChanges {
                target: orangeBar
                height: lineHeigth
            }
        }
    ]
}

