import QtQuick 2.0
import Widgets 0.1

Rectangle {
    id: indicatorRow

    property QtObject currentItem
    property int currentItemIndex: currentItem ? currentItem.ownIndex : -1
    property alias row: row
    property IndicatorsModel indicatorsModel: null
    property int rowWidth: width

    // alerts
    property bool messagesAlert: false
    onMessagesAlertChanged: changeAlert("Messages", messagesAlert)

    Component.onCompleted: {
        row.calculateItemSpacing()
    }

    Behavior on y {NumberAnimation {duration: 300; easing.type: Easing.OutCubic} }

    color: "black"
    width: units.dp(320)
    height: units.dp(24)

    function changeAlert(groupName, alert) {
        var i, child
        for (i=0; i<row.children.length; i++) {
            child = row.children[i]
            if (child === rowRepeater) continue;
            if (child.group == groupName) {
                child.alert = alert
            }
        }
    }

    Row {
        id: row

        width: parent.rowWidth
        height: parent.height
        anchors.right: parent.right
        property int additionalSpacing: 0
        function calculateItemSpacing() {
            additionalSpacing = (indicatorRow.rowWidth - row.childrenRect.width) / indicatorsModel.itemCount;
        }

        Repeater {
            id: rowRepeater
            model: indicatorsModel ? indicatorsModel : undefined
            IndicatorGroup {
               id: indicatorGroup

               property int ownIndex: index

               group: groupName
               model: attributes
               isCurrentItem: indicators.state == "reveal" ? ownIndex == indicatorRow.currentItemIndex : false
               height: indicatorRow.height
               additionalSpacing: row.additionalSpacing
               y: {
                   if (shell.tablet || initial || indicators.state == "reveal" || indicators.state == "hint") {
                       return 0
                   } else {
                       return -indicatorRow.height
                   }
               }
               Behavior on y {
                    NumberAnimation{
                        duration: {
                            if (index == 0) {
                                return 250
                            } else if (index == 1) {
                                return 150
                            } else if (index == 2) {
                                return 550
                            } else if (index == 3) {
                                return 400
                            } else {
                                return Math.random() * 200
                            }
                        }
                        easing.type: Easing.OutCubic
                    }
                }
            }
        }
    }
}
