import QtQuick 2.0
import Widgets 0.1
import "../Components"

Showable {
    id: indicators

    property int rowWidth: units.dp(320)
    property int progress: 0
    property int openedHeight: units.dp(854/1.5)
    property int separatorLineHeight: units.dp(4/1.5)
    property int topbarHeight: units.dp(24)
    property variant activeMenu

    property int hintValue
    property int lockThreshold: height/2

    property Revealer revealer: null

    onProgressChanged: {
        var contentProgress = progress - handle.height
        if (!showAnimation.running && !hideAnimation.running && !revealer.hintingAnimation.running) {
            if (contentProgress <= hintValue && indicators.state == "reveal") {
                indicators.state = "hint"
                menuContent.hideAll()
            } else if (contentProgress > hintValue && contentProgress < lockThreshold) {
                menuContent.showMenu()
                indicators.state = "reveal"
            } else if (contentProgress >= lockThreshold) {
                indicators.state = "locked"
            }
        }
    }

    state: "initial"
    onStateChanged: {
        if (state == "reveal") calculateCurrentItem(revealer.lateralPosition, false)
    }

    function newMessageData(data) {
        menuContent.getMenu("Messages").newMessage(data)
    }

    function handlePress() {
        menuContent.hideAll()
        indicators.state = "hint"
    }

    function openOverview() {
        menuContent.showOverview()
    }

    function calculateCurrentItem(xValue, useBuffer) {
        var rowCoordinates
        var itemCoordinates
        var currentItem
        var bufferThreshold = 0.4 // percentage of icon width
        var distanceFromRightEdge
        var bufferExceeded = false

        if (indicators.state == "commit" || indicators.state == "locked" || showAnimation.running || hideAnimation.running) return

        // trace the container item in indicatorRow that corresponds to current x value. Y is constant.
        rowCoordinates = mapToItem(indicators, xValue, 0);
        // get the current delegate
        currentItem = indicatorRow.row.childAt(rowCoordinates.x, 0);
        if (currentItem && currentItem != indicatorRow.currentItem ) {
            itemCoordinates = indicatorRow.mapToItem(currentItem, rowCoordinates.x, 0);
            distanceFromRightEdge = (currentItem.width - itemCoordinates.x) / (currentItem.width)
            if (Math.abs(currentItem.ownIndex - indicatorRow.currentItemIndex) > 1) {
                bufferExceeded = true
            } else {
                if (indicatorRow.currentItemIndex < currentItem.ownIndex && distanceFromRightEdge < bufferThreshold) {
                    bufferExceeded = true
                } else if (indicatorRow.currentItemIndex > currentItem.ownIndex && distanceFromRightEdge > (units.dp(1/1.5) - bufferThreshold)) {
                    bufferExceeded = true
                }
            }
            if ((!useBuffer || (useBuffer && bufferExceeded)) || indicatorRow.currentItem < 0 || indicatorRow.currentItem == null)  {
                indicatorRow.currentItem = currentItem;
            }
        }
    }
    // eater
    MouseArea {
        anchors {
            top: parent.top
            bottom: handle.bottom
            left: parent.left
            right: parent.right
        }
    }
    MenuContent {
        id: menuContent

        width: parent.rowWidth
        anchors.right: parent.right
        anchors.top: indicatorRow.bottom
        anchors.bottom: handle.top
        focusedIndicator: indicatorRow.currentItem ? indicatorsModel.get(indicatorRow.currentItemIndex).groupName : ""
        y: indicators.state == "locked" || indicators.state == "commit" ? 0 : units.dp(20)
        animate: indicators.state == "reveal"
        Behavior on y {NumberAnimation {duration: 300; easing.type: Easing.OutCubic}}

    }

    Item {
        id: handle

        y: progress - height
        anchors {
            right: parent.right
        }
        width: parent.rowWidth
        height: handleImage.height + orangeLine.height

        Rectangle {
            width: parent.width
            height: units.dp(4/1.5)
            gradient: Gradient {
                GradientStop { position: 0.0; color: "transparent" }
                GradientStop { position: 1.0; color: "black" }
            }
            anchors.bottom: handleImage.top
            opacity: 0.4
        }

        ImageMipmap {
            id: handleImage
            resource: "graphics/handle.png"
            height: topbarHeight
            width: parent.width
        }

        ImageMipmap {
            id:  orangeLine
            anchors.top: handleImage.bottom
            resource: "graphics/orange_key_line.png"
            width: parent.width
            height: separatorLineHeight
        }
    }

    Rectangle {
        id: background
        anchors {
            left: parent.left
            right: indicatorRow.left
            top: parent.top
        }
        height: topbarHeight

        color: "black"
    }

    ImageMipmap {
        id: staticOrangeLine
        anchors.top: background.bottom
        anchors.left: parent.left
        resource: "graphics/orange_key_line.png"
        width: parent.width - rowWidth
        height: separatorLineHeight
    }

    IndicatorRow {
        id: indicatorRow
        width: rowWidth
        rowWidth: parent.rowWidth
        anchors.right: parent.right
        height: indicators.topbarHeight
        y: indicators.state == "initial" || indicators.state == "hint" || indicators.state == "reveal" ? 0 : -height

        Behavior on y {NumberAnimation {duration: 300; easing.type: Easing.OutCubic} }
        indicatorsModel: indicatorsModel
        messagesAlert: menuContent.messagesMenu ? menuContent.messagesMenu.messageCount > 0 : false
    }

    IndicatorsModel {
        id: indicatorsModel
    }

    Connections {
        target: hideAnimation
        onRunningChanged: {
            if (hideAnimation.running) {
                indicators.state = "initial"
                menuContent.hideAll()
            }
        }
    }
    Connections {
        target: showAnimation
        onRunningChanged: {
            if (indicators.state == "hint") menuContent.showOverview()
            if (showAnimation.running) indicators.state = "commit"
        }
    }

    Connections {
        target: revealer
        onLateralPositionChanged: {
            var buffer = revealer.dragging ? true : false
            indicators.calculateCurrentItem(revealer.lateralPosition, buffer)
        }
    }

    states: [
        State {
            name: "initial"
        },
        State {
            name: "hint"
        },
        State {
            name: "reveal"
        },
        State {
            name: "locked"
        },
        State {
            name: "commit"
        }
    ]
}
