import QtQuick 2.0
import Widgets 0.1
import "Menus"

Rectangle {
    id: content

    // menus
    property variant locationMenu
    property variant bluetoothMenu
    property variant syncMenu
    property variant messagesMenu
    property variant networkMenu
    property variant overviewMenu
    property variant soundMenu
    property variant timeMenu
    property variant batteryMenu

    property bool animate: true
    property bool __overview: true // "state of the menu"
    property bool __shown: false

    property int rowWidth: units.dp(320)
    width: units.dp(320)
    height: units.dp(500/1.5)
    color: "#282421"

    clip: true
    property string focusedIndicator: ""
    onFocusedIndicatorChanged: __loadMenu(focusedIndicator)

    Component.onCompleted: {
        locationMenu = getMenu("Location")
        bluetoothMenu = getMenu("Bluetooth")
        syncMenu = getMenu("Sync")
        messagesMenu = getMenu("Messages")
        networkMenu = getMenu("Network")
        overviewMenu = getMenu("Overview")
        soundMenu = getMenu("Sound")
        timeMenu = getMenu("Time")
        batteryMenu = getMenu("Battery")
    }


    function getMenu(menu) {
        if (menu == "Overview") return overview

        var menuWindow = __getMenuWindow(menu)
        if (menuWindow == undefined) {
            //console.log("Warning: \"" + menu + "\" not found!")
        }
        return menuWindow
    }

    function showMenu() {
        __loadMenu(focusedIndicator)
        __shown = true
        __overview = false
    }

    function showOverview() {
        __shown = true
        __overview = true
    }

    function hideAll() {
        __shown = false
    }

    function __loadMenu(menuFile) {

        var menu = content.__getMenuWindow(menuFile)

        if (menu == undefined) {
            //console.log("Warning: \"" + menuFile + "\" doesn't exist. No menu loaded!")
            return;
        }

        if (indicators.activeMenu) {
            indicators.activeMenu.shown = false
        }
        indicators.activeMenu = menu
        indicators.activeMenu.shown = true
    }

    function __getMenuWindow(menu) {
        var i, child
        for (i=0; i<menus.children.length; i++) {
            child = menus.children[i]
            if (child === menuRepeater) continue;
            if (child.item.groupName == menu) {
                menus.index = i
                return child.item
            }
        }
    }

    Item {
        id: header
        anchors {
            left: parent.left
            right: parent.right
        }
        height: units.dp(85/1.5)
        opacity: __shown ? 1 : 0
        Behavior on opacity {NumberAnimation{duration: 100}}

        Text {
            text: content.__overview ? "System" : indicatorsModel.get(menus.index).groupName
            color: "white"
            font.pixelSize: units.dp(30)
            anchors {
                top: parent.top
                topMargin: units.dp(10)
                left: parent.left
                leftMargin: units.dp(20/1.5)
            }
        }
    }

    Row {
        id: menus

        property int index: 0

        anchors.top: header.bottom
        width: childrenRect.width
        height: indicators.openedHeight - handle.height - header.height
        spacing: -units.dp(380/1.5)
        x: - index* (content.width + spacing)
        opacity: !__overview && __shown ? 1 : 0

        Behavior on x {NumberAnimation {duration: animate ? 300 : 0; easing.type: Easing.OutQuad}}
        Behavior on opacity {
            SequentialAnimation{
                PauseAnimation {duration: 300}
                NumberAnimation{duration: 200}
            }
        }

        Repeater {
            id: menuRepeater

            model: indicatorsModel
            delegate: Loader {
                width: content.width
                height: menus.height
                source: "Menus/%1.qml".arg(groupName)

                onLoaded: item.groupName = groupName
                onHeightChanged: if (item) item.height = height
                onWidthChanged: if (item) item.width = width
            }
        }
    }

    Overview {
        id: overview

        anchors.top: header.bottom
        width: content.width
        height: indicators.openedHeight - handle.height -  header.height
        opacity: content.__overview && content.__shown ? 1 : 0
        Behavior on opacity {NumberAnimation{duration: 200}}

        onMenuSelected: {
            __loadMenu(menuFile)
            content.__overview = false
        }
    }
}
