import QtQuick 2.0
import "../Components"
import Widgets 0.1

Item {
    id: launcher

    property ListModel model: ListModel {}
    property ListModel applicationsRunning: ListModel {}
    property ListModel gadgetsRunning: ListModel {}
    property int lastFavouriteIndex: -1

    property bool gadgetsEnabled: false
    property bool interactive: true // can be used to disable all interactions
    property bool applicationFocused: false

    property int progress
    progress: {
        if (launcher.state == "spreadMoving")
            return mouseArea.mouseX
        else if (launcher.state == "spread")
            return launcher.width
        else
            return 0
    }

    // Thresholds and dimensions
    property int shortcutsHintWidth: 10
    property int spreadHintWidth: 50
    property int shortcutsWidth: 110
    property int shortcutsThreshold: units.dp(25/1.5)
    property int shortcutsCloseThreshold: units.dp(10/1.5)
    property int spreadHintThreshold: shortcutsWidth
    property int spreadOutThreshold: 320
    property int gadgetsSpreadWidth: 200

    property string iconPath: "."
    property string appScreenshotPath: "."
    property string gadgetScreenshotPath: "."


    property string currentIconSource: shortcuts.shortcutsModel.get(shortcuts.currentIndex) ? '../' + iconPath + "/" + shortcuts.shortcutsModel.get(shortcuts.currentIndex).identifier + '.png': ""
    property string currentItemText: shortcuts.shortcutsModel.get(shortcuts.currentIndex) ? shortcuts.shortcutsModel.get(shortcuts.currentIndex).name : ""
    property int labelPosition: shortcuts.labelPosition
    property int labelHeight: shortcuts.iconWidth
    property bool labelVisible: mouseArea.pressed && (launcher.state == "shortcuts" || launcher.state == "spreadHint")

    property int launcherProgressValue
    launcherProgressValue: {
        if (launcher.state == ""
                || launcher.state == "hint"
                || launcher.state == "moving"
                || launcher.state == "shortcuts") {
            return 0
        } else if (launcher.state == "spreadHint") {
            return 300 //launcher.shortcutsWidth
        } else if (launcher.state == "spreadMoving") {
            return 400//mouseArea.mouseX
        } else if (launcher.state == "spread") {
            return 500 //launcher.width
        }
    }

    signal launcherItemSelected(int itemIndex)
    signal launcherApplicationSelected(string name)
    signal launcherGadgetSelected(string name)
    signal dashItemSelected(int itemIndex)
    signal applicationClosed(int index, string name, string type)
    signal allClosed()

    signal moving()
    signal dash()
    signal begin()

    // internal properties hidden from public api
    QtObject {
        id: priv

        property bool shortcutAnimationOngoing
        property string previousStateHelp
        property string previousState: ""
    }

    onStateChanged: {
        priv.previousState = priv.previousStateHelp
        priv.previousStateHelp = state
        if (launcher.state != "shorcuts") {
            dismissTimer.stop()
        }

        if (launcher.state == "spread") {
            mouseArea.visible = false
            hideTimer.restart()
            mouseArea.visible = true
        }
        //console.log("Launcher state: " + state)
    }

    function add( delta ) {
        var item
        var index = -1
        for ( var i = 0, l = launcher.model.count; i < l; i ++ ) {
            item = launcher.model.get( i )
            if ( item.name == delta.name ) index = i
            if ( item.name == delta.name && item.favourite ) delta.favourite = true
            if ( item.favourite ) launcher.lastFavouriteIndex = i
        }
        if ( delta.favourite && index == -1 ) {
            launcher.model.append( delta )
        }
        else if ( delta.favourite && index > -1 ) {
            launcher.model.set( index, { 'running' : true } )
        }
        else if ( !delta.favourite && index == -1 ) {
            if ( delta.name == 'dash' ) launcher.model.insert( 0, delta )
            else launcher.model.insert( launcher.lastFavouriteIndex + 1, delta )
        }
        else if ( !delta.d && index > -1 ) {
            launcher.model.move( index, launcher.lastFavouriteIndex + 1, 1 )
        }
        if ( delta.running ) {
            if ( delta.type == 'application' || !launcher.gadgetsEnabled) {
                for ( i = 0, l = launcher.applicationsRunning.count; i < l; i ++ ) {
                    if ( launcher.applicationsRunning.get( i ).name == delta.name ) {
                        launcher.applicationsRunning.move( i, 0, 0 )
                        return
                    }
                }
                if ( delta.name != 'dash' ) {
                    launcher.applicationsRunning.insert( 0, delta )
                }
            } else if ( delta.type == 'gadget' ) {
                for ( i = 0, l = launcher.gadgetsRunning.count; i < l; i ++ ) {
                    if ( launcher.gadgetsRunning.get( i ).name == delta.name ) {
                        launcher.gadgetsRunning.move( i, 0, 0 )
                        return
                    }
                }
                if ( delta.name != 'dash' ) {
                    launcher.gadgetsRunning.insert( 0, delta )
                }
            } else {
                if ( delta.name != 'dash' ) {
                    //console.log("Unknown application type", delta.type, "for app", delta.name)
                }
            }
        }
        shortcuts.init()
    }

    function removeAll() {
        var item

        while ( launcher.applicationsRunning.count > 0 ) {
            launcher.applicationsRunning.remove( launcher.applicationsRunning.count - 1 )
        }

        while ( launcher.gadgetsRunning.count > 0 ) {
            launcher.gadgetsRunning.remove( launcher.gadgetsRunning.count - 1 )
        }

        for ( var i = 0; i < launcher.model.count;) {
            item = launcher.model.get( i )
            if ( !item.favourite ) {
                launcher.model.remove( i )
            } else {
                 i++
            }
        }

        for ( var i = 0; i < launcher.model.count; i++) {
            item = launcher.model.get( i )
            if ( item.favourite && item.running ) {
                launcher.model.setProperty( i, 'running', false )
            }
        }
        shortcuts.init()
    }

    function remove( name ) {
        var item
        for ( var i = 0, l = launcher.applicationsRunning.count; i < l; i ++ ) {
            if ( launcher.applicationsRunning.get( i ).name == name ) {
                launcher.applicationsRunning.remove( i )
                break
            }
        }
        for ( var i = 0, l = launcher.gadgetsRunning.count; i < l; i ++ ) {
            if ( launcher.gadgetsRunning.get( i ).name == name ) {
                launcher.gadgetsRunning.remove( i )
                break
            }
        }
        for ( var i = 0, l = launcher.model.count; i < l; i ++ ) {
            item = launcher.model.get( i )
            if ( item.name == name && !item.favourite ) {
                launcher.model.remove( i )
                break
            }
            else if ( item.name == name && item.favourite && item.running ) {
                launcher.model.setProperty( i, 'running', false )
                break
            }
        }
        shortcuts.init()
    }

    function hide() {
        mouseArea.visible = false
        launcher.state = ""
        mouseArea.visible = true
    }

    function calculateCurrentShortcut() {
        var shortcutObject = mapToItem(shortcuts, mouseArea.mouseX, mouseArea.mouseY)
        shortcuts.calculateHighlight(shortcutObject.x, shortcutObject.y)
    }

    Rectangle {
        id: backgroundShade
        anchors.fill: parent
        color: "black"
        opacity: {
            if (launcher.state == "" || launcher.state == "hint" || launcher.state == "moving") {
                return 0
            } else return 0.4
        }
        visible: opacity != 0

        Behavior on opacity { NumberAnimation { duration: 200; easing.type: Easing.OutQuad} }

        MouseArea {
            anchors.fill: parent
            onClicked: {
                launcher.state = ""
            }
            visible: launcher.state != "spread"
        }
    }

    LauncherShortcuts {
        id: shortcuts
        width: shortcutsWidth
        anchors {
            top: parent.top
            bottom: parent.bottom
        }
        x: -width
        model: launcher.model
        onApplicationSelected: {
            launcher.state = ""
            launcher.shortcutsApplicationLaunched()
        }
        highlightVisible: mouseArea.pressed && (launcher.state == "shortcuts" || launcher.state == "spreadHint")
        iconPath: launcher.iconPath

        Behavior on x { NumberAnimation {duration: launcher.state == "hint" ? 70 : launcher.state == "moving" ? 0 : 300; easing.type: Easing.OutCubic } }
    }

    DraggingArea {
        id: mouseArea

        property int initialX
        property int movingDelta: initialX - launcher.shortcutsHintWidth
        zeroVelocityCounts: true
        orientation: Qt.Horizontal
        onDragVelocityChanged: if (dragVelocity < -700 && launcher.state == "shortcuts") launcher.hide()
        visible: launcher.interactive
        width: {
            if (launcher.state == "") {
                return shortcutsThreshold
            } else {
                return shortcutsWidth
            }
        }
        height: launcher.height

        onPressed: {
            initialX = mouseX
            if (launcher.state == "") launcher.state = "hint"
            launcher.calculateCurrentShortcut()
            dismissTimer.stop()

            if (launcher.state == "shortcuts" && !shortcuts.animating) {
                shortcuts.unfold()
            }
        }
        onPositionChanged: {
            launcher.calculateCurrentShortcut()
            if (launcher.state == "shortcuts" && mouseX < shortcutsCloseThreshold) {
                launcher.state = ""
            } else if (launcher.state == "hint" && mouseX > initialX ) {
                launcher.state = "moving"
            } else if (mouseX > shortcutsThreshold && mouseX < spreadHintThreshold) {
                launcher.state = "shortcuts"
            } else if (mouseX >= spreadHintThreshold && mouseX < shortcutsWidth) {
                if (launcher.applicationFocused){
                    launcher.state = "spreadHint"
                }
            } else if (mouseX >= shortcutsWidth && mouseX < spreadOutThreshold) {
                if (launcher.applicationFocused) {
                    launcher.state = "spreadMoving"
                } else {
                    dismissTimer.restart()
                }
            } else if (mouseX >= spreadOutThreshold && launcher.applicationFocused) {
                launcher.state = "spread"
            }

            var shortcutObject = mapToItem(shortcuts, mouseX, mouseY)
            shortcuts.calculateHighlight(shortcutObject.x, shortcutObject.y)
        }

        // launcher scrolling controls here
        onMouseYChanged: {
            var scrollAreaHeight = shortcuts.itemWidth/2
            var maxScrollSpeed = 20
            if (launcher.state == "shortcuts" && !priv.shortcutAnimationOngoing) {
                if (mouseY < scrollAreaHeight) {
                    shortcuts.scrollValue = maxScrollSpeed - mouseY / (scrollAreaHeight/maxScrollSpeed)
                    if (shortcuts.open && !shortcuts.animating) {
                        shortcuts.scroll()
                    }
                } else if (mouseArea.height - mouseY < scrollAreaHeight) {
                    shortcuts.scrollValue = -maxScrollSpeed + (mouseArea.height - mouseY) / (scrollAreaHeight/maxScrollSpeed)
                    if (shortcuts.open && !shortcuts.animating) {
                        shortcuts.scroll()
                    }
                } else {
                    shortcuts.stopScroll()
                }
            }
        }

        onReleased: {
            var currentApplicationIndex
            if (launcher.state == "shortcuts" && !priv.shortcutAnimationOngoing) {
                // launch app
                currentApplicationIndex = shortcuts.selectCurrentApplication()
                if (currentApplicationIndex === 0) {
                    launcher.dashItemSelected(0)
                    launcher.state = ""
                } else if (currentApplicationIndex === -1) {
                    launcher.state = "shortcuts"
                } else if (Math.abs(dragVelocity) < 100) {
                    currentApplicationIndex-- // due to dash iconPath
                    launcherApplicationSelected(shortcuts.model.get(currentApplicationIndex).identifier)
                    launcher.state = ""
                }
            }

            if (launcher.state == "hint" || launcher.state == "moving") {
                launcher.state = ""
            } else if (launcher.state == "spreadMoving" || launcher.state == "spreadHint") {
                if (dragVelocity > 800) {
                    launcher.state = "spread"
                } else {
                    launcher.state = "shortcuts"
                }
            }

            shortcuts.resetHighlight()
            shortcuts.stopScroll()

            //unfoldTimer.stop()

            if (launcher.state == "shortcuts") {
                dismissTimer.restart()
            }

        }
    }

    Timer {
        id: dismissTimer
        interval: 8000;
        running: false;
        repeat: false
        onTriggered: {
            launcher.state = ""
        }
    }

    Timer {
        id: hideTimer
        interval: 50;
        running: false;
        repeat: false
        onTriggered: {
            launcher.state = ""
        }
    }

    Timer {
        id: unfoldTimer
        interval: 150;
        running: false;
        repeat: false
        onTriggered: {
            shortcuts.unfold()
        }
    }

    states: [
        State {
            name: "hint"
            PropertyChanges {
                target: shortcuts
                x: -shortcuts.width + launcher.shortcutsHintWidth
            }
        },
        State {
            name: "moving"
            PropertyChanges {
                target: shortcuts
                x: -shortcuts.width + mouseArea.mouseX - mouseArea.movingDelta
            }
        },
        State {
            name: "shortcuts"
            PropertyChanges {
                target: shortcuts
                x: 0
            }
        },
        State {
            name: "spreadHint"
            extend: "shortcuts"
        },
        State {
            name: "spreadMoving"
            PropertyChanges {
                target: launcher
                launcherProgressValue: mouseArea.mouseX
            }
            PropertyChanges {
                target: shortcuts
                x: 0
            }
        },
        State {
            name: "spread"
            extend: "shortcuts"
        }

    ]

    transitions: [
        Transition {
            to: ""
            SequentialAnimation {
                PropertyAnimation {
                    target: shortcuts
                    properties: "x"
                    duration: 400
                    easing.type: Easing.OutCubic
                }
                ScriptAction {
                    script: {
                        shortcuts.fold()
                    }
                }
            }
        },
        Transition {
            to: "hint"
            PropertyAnimation {
                target: shortcuts
                properties: "x"
                duration: 100
                easing.type: Easing.OutCubic
            }
        },
        Transition {
            to: "shortcuts"
            SequentialAnimation {
                ScriptAction {
                    script: {
                        if (priv.previousState == "moving") {
                            priv.shortcutAnimationOngoing = true
                        } else {
                            launcher.begin()
                        }

                        unfoldTimer.start()
                    }
                }
                ParallelAnimation {
                    SequentialAnimation {
                        PropertyAnimation {
                            target: shortcuts
                            properties: "x"
                            duration: 300
                            easing.type: Easing.OutCubic
                        }
                        ScriptAction {
                            script: {
                                priv.shortcutAnimationOngoing = false
                            }
                        }
                    }
                }
            }
        },
        Transition {
            to: "spreadHint"
            PropertyAnimation {
                target: shortcuts
                properties: "x"
                duration: 300
                easing.type: Easing.OutCubic
            }
        },
        Transition {
            to: "spreadMoving"
            ScriptAction {
                script: {
                    priv.shortcutAnimationOngoing = false
                    launcher.moving()
                }
            }
        },
        Transition {
            to: "spread"
            ScriptAction {
                script: {
                    launcher.dash()
                }
            }
        }
    ]
}
