import QtQuick 2.0

Item {
    id: shortcuts

    property ListModel model
    property ListModel shortcutsModel: ListModel{}
    property ListModel openModel: ListModel{}
    property ListModel semiFoldedModel: ListModel{}
    property ListModel foldedModel: ListModel{}

    property bool enabled: true
    property bool highlightVisible: false
    property string iconPath: "."

    property bool animating: openShortcuts.animating || semiFoldedShortcut.animating || foldedShortcuts.animating // read only
    property int currentIndex: -1

    property int scrollValue: 0

    property int itemWidth: width
    property int iconWidth: 0.9 * itemWidth
    property int iconCount: shortcutsModel.count

    property int foldedSize: itemWidth - foldedShortcuts.initialFoldedOffset

    property int openItemsCount
    property int foldedItemsCount: iconCount - openItemsCount
    property int fillItemHeight: height - openItemsCount * itemWidth - foldedItemsCount * foldedSize
    property bool open: false
    property bool inverted: true
    property int labelPosition
    labelPosition: {
        if (!inverted) {
            return currentIndex*shortcuts.itemWidth + shortcuts.itemWidth/2 + shortcutsContainer.targetY
        } else {
            return shortcuts.height - (currentIndex*shortcuts.itemWidth + shortcuts.itemWidth/2 + shortcutsContainer.targetY)
        }
    }

    rotation: inverted ? 180 : 0

    QtObject {
        id: priv

        property int modelCount: shortcuts.model ? shortcuts.model.count : 0
    }

    signal applicationSelected();

    Component.onCompleted: init()

    function init() {
        clearModels()
        addDashIcon()
        filterModel()
        calculateOpenIcons()
        createModels()
    }

    function clearModels() {
        shortcutsModel.clear()
        openModel.clear()
        semiFoldedModel.clear()
        foldedModel.clear()
    }

    function filterModel() {
        for (var i = 0; i < shortcuts.model.count; i++) {
            var item = shortcuts.model.get(i);
            if (item.favourite || item.running) {
                shortcutsModel.append(shortcuts.model.get(i));
            }
        }
    }

    function addDashIcon() {
        shortcutsModel.append({"identifier": "dash",
                               "favourite": true,
                               "running": false,
                               "name": "Dash"});
    }

    function calculateOpenIcons() {
        var foldedSize = (shortcuts.itemWidth - foldedShortcuts.initialFoldedOffset)
        var maxopenItemsCount = Math.floor(shortcuts.height / shortcuts.itemWidth)
        var leftOverIcons = shortcuts.iconCount - maxopenItemsCount
        var leftOverSpace
        if (leftOverIcons > 0) {
            leftOverSpace = shortcuts.height - shortcuts.itemWidth * maxopenItemsCount
            leftOverIcons = leftOverIcons - Math.floor(leftOverSpace / foldedSize)
            while (leftOverIcons > 0) {
                maxopenItemsCount--
                leftOverIcons = shortcuts.iconCount - maxopenItemsCount
                if (leftOverIcons > 0) {
                    leftOverSpace = shortcuts.height - shortcuts.itemWidth * maxopenItemsCount
                    leftOverIcons = leftOverIcons - Math.floor(leftOverSpace / foldedSize)
                }
            }
        }
        shortcuts.openItemsCount = Math.min(maxopenItemsCount, shortcuts.iconCount)
        shortcuts.visible = true
    }

    function createModels() {
        for (var i = 0; i < shortcutsModel.count; i++) {
            if (i < shortcuts.openItemsCount) {
                shortcuts.openModel.append(shortcutsModel.get(i))
            } else if (i == shortcuts.openItemsCount) {
                shortcuts.semiFoldedModel.append(shortcutsModel.get(i))
            } else {
                shortcuts.foldedModel.append(shortcutsModel.get(i))
            }
        }
    }

    function unfold() {
        if (!shortcuts.open) {
            var addvalue = 0
            if (shortcuts.currentIndex - shortcuts.openModel.count >= 0) {
                addvalue = -2 * shortcuts.itemWidth / 3 + shortcuts.fillItemHeight
            }
            if (shortcuts.currentIndex - shortcuts.openModel.count > 0) {
                addvalue += -2 * shortcuts.itemWidth / 3 * (shortcuts.currentIndex - shortcuts.openModel.count)
            }

            shortcutsContainer.targetY += addvalue
            shortcuts.open = true
        }
    }

    function fold() {
        shortcutsContainer.targetY = 0
        shortcuts.open = false
    }

    function scroll() {
        shortcutsContainer.animateY = false
        if (!scrollTimer.running) {
            scrollTimer.start()
        }
    }

    function stopScroll() {
        scrollTimer.stop()
        shortcutsContainer.animateY = true
    }

    function calculateScrollingHighlight() {
        var posX
        var posY
        if (scrollTimer.running) {
            var scrollValue = inverted ? -shortcuts.scrollValue : shortcuts.scrollValue
            if (scrollValue < 0) {
                posX = Math.round(itemWidth/2)
                posY = Math.round(shortcuts.height - itemWidth/2)
            } else if (scrollValue > 0) {
                posX = Math.round(itemWidth/2)
                posY = Math.round(itemWidth/2)
            }
            calculateHighlight (posX, posY)
        }

    }

    function calculateHighlight(posX, posY) {
        var container = shortcutsContainer.childAt(shortcutsContainer.width/2, posY - shortcutsContainer.y)
        var targetObject
        if (container) {
            var posObject = shortcuts.mapToItem(container, container.width/2, posY)
            targetObject = container.childAt(posObject.x, posObject.y)
            if (targetObject) {
                if (targetObject.ownIndex !== undefined) {
                    shortcuts.currentIndex = targetObject.ownIndex
                }
            }
        } else {
            shortcuts.resetHighlight()
        }
    }

    function resetHighlight() {
        shortcuts.currentIndex = -1
    }

    function selectCurrentApplication() {
        if (currentIndex < shortcuts.openItemsCount || foldedShortcuts.foldedAngle == 0) {
            return currentIndex
        } else {
            return -1
        }
    }

    Image {
        id: background
        source: "graphics/launcher_bg.png"
        width: parent.width
        x: 0
        rotation: inverted ? 180 : 0
        height: parent.height
    }

    Component {
        id: shortcutsItemDelegate
        Item {
            id: delegateItem
            property int ownIndex: parent ? index + parent.initialIndex : -1
            property bool highlighted: ownIndex == shortcuts.currentIndex

            z: -index
            anchors.horizontalCenter: parent ? parent.horizontalCenter : undefined
            width: shortcuts.itemWidth
            height: width

            Item {
                width: shortcuts.iconWidth
                height: width
                anchors.centerIn: parent
                transform: Rotation { origin.x: width/2; origin.y: height; axis { x: 1; y: 0; z: 0 } angle: parent ? parent.foldedAngle : 0 }
                smooth: true
                Image {
                   id: highlight
                   source: "graphics/highlight.png"
                   anchors.fill: parent
                   anchors.margins: -2
                   smooth: true
                   opacity: delegateItem.highlighted && shortcuts.highlightVisible ? 1 : 0
                   Behavior on opacity { NumberAnimation { duration: 100 } }
                }
                Rectangle {
                    id: itemBackground
                    color: parent.highlighted ? "transparent" : "transparent"
                    anchors.fill: parent
                    radius: 6
                    Behavior on color { ColorAnimation { duration: 100 } }

                    Image {
                        id: runningIndicator

                        source: "graphics/triangle.png"
                        anchors {
                            right: inverted ? undefined : parent.left
                            left: inverted ? parent.right : undefined
                            verticalCenter: parent.verticalCenter
                            margins: -1
                        }
                        rotation: inverted ? 180 : 0
                        visible: false
                        opacity: 0.8
                    }
                }
                Image {
                    id: image
                    anchors.centerIn: parent
                    source:  '../' + shortcuts.iconPath + "/" + identifier + '.png'
                    smooth: true
                    width: shortcuts.itemWidth
                    height: width
                    rotation: inverted ? 180 : 0
                }

            }
            MouseArea {
                anchors.fill: parent
                onClicked: {
                }
            }
        }
    }

    Item {
        id: shortcutsContainer

        property bool animateY: true
        property real targetY

        x: ( parent.width - background.width) / 2
        y: targetY
        width: shortcuts.width
        height: openShortcuts.height + semiFoldedShortcut.height + foldedShortcuts.height

        Behavior on y { NumberAnimation{duration: shortcutsContainer.animateY ? 200 : 0; } }

        onYChanged: {
            calculateScrollingHighlight()
        }

        ShortcutsContainer {
            id: openShortcuts
            objectName: "openShortcuts"

            width: shortcuts.width
            height: children.height
            z: 3
            model: shortcuts.openModel
            delegate: shortcutsItemDelegate
            initialIndex: 0
        }

        ShortcutsContainer {
            id: semiFoldedShortcut
            objectName: "semiFoldedShortcut"

            foldedAngle: shortcuts.open ? 0 : 90 * (height - initialFoldedOffset)/height - 80 // todo: better algorithm here
            initialFoldedOffset: -2 * shortcuts.itemWidth / 3 + shortcuts.fillItemHeight
            foldedOffset: shortcuts.open ? 0 : initialFoldedOffset
            visible: shortcuts.iconCount > shortcuts.openItemsCount
            width: shortcuts.itemWidth
            height: children.height
            anchors.top: openShortcuts.bottom
            z: 2
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.topMargin: foldedOffset
            model: shortcuts.semiFoldedModel
            delegate: shortcutsItemDelegate
            initialIndex: shortcuts.openModel.count
        }

        ShortcutsContainer {
            id: foldedShortcuts
            objectName: "foldedShortcuts"

            foldedAngle: shortcuts.open ? 0 : 68
            initialFoldedOffset: 2 * shortcuts.itemWidth / 3
            foldedOffset: shortcuts.open ? 0 : initialFoldedOffset
            width: shortcuts.width
            height: children.height
            spacing: -foldedOffset
            anchors.top: semiFoldedShortcut.bottom
            anchors.topMargin: -foldedOffset
            z: 1
            model: shortcuts.foldedModel
            delegate: shortcutsItemDelegate
            initialIndex: shortcuts.openModel.count + shortcuts.semiFoldedModel.count
        }
    }

    Timer {
        id: scrollTimer

        repeat: true
        running: false
        interval: 5
        onTriggered:{
            if (shortcutsContainer.height > shortcuts.height) {
                var scrollValue
                if (inverted) {
                    scrollValue = -shortcuts.scrollValue
                } else {
                    scrollValue = shortcuts.scrollValue
                }

                var newValue = shortcutsContainer.targetY + scrollValue

                if (newValue > 0) {
                    newValue = 0
                } else if (newValue < shortcuts.height - shortcutsContainer.height) {
                    newValue = shortcuts.height - shortcutsContainer.height
                }
                shortcutsContainer.targetY = newValue
            }
        }
    }
}
