import QtQuick 2.0
import Widgets 0.1

Item {
    id: actionButton

    property string label

    property string buttonGraphic: ""
    property string pressedGraphic: ""
    property string hoverGraphic: ""

    // these are used if no buttonGraphic is available
    property color fillColor: "transparent"
    property real fillAlpha: 0.3
    property color borderColor: Qt.rgba(1.0, 1.0, 1.0, actionButton.fillAlpha)

    property int touchAreaMargins: 0

    signal selected()

    states: [
        State {
            name: "hover"
            PropertyChanges {
                target: actionButton
                fillAlpha: 0.5
            }
        },
        State {
            name: "down"
            PropertyChanges {
                target: actionButton
                fillAlpha: 1
            }
        }
    ]

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        anchors.margins: touchAreaMargins
        hoverEnabled: true
        onEntered: actionButton.state = "hover";
        onExited: actionButton.state = ""
        onPressed: actionButton.state = "down"
        onReleased: {
            if (containsMouse) {
                actionButton.selected()
                actionButton.state = "hover"
            }
        }
    }

    BorderImageMipmap {
        id: image
        anchors.fill: parent
        border { left: units.dp(10/1.5); top: units.dp(10/1.5); right: units.dp(10/1.5); bottom: units.dp(10/1.5) }
        horizontalTileMode: BorderImage.Stretch
        verticalTileMode: BorderImage.Stretch
        resource: {
            if (actionButton.state == "down") {
                return actionButton.pressedGraphic != "" ? actionButton.pressedGraphic : actionButton.buttonGraphic
            }
            else if (actionButton.state == "hover") {
                return actionButton.hoverGraphic != "" ? actionButton.hoverGraphic : actionButton.buttonGraphic
            }
            else {
                return actionButton.buttonGraphic
            }
        }
    }

    Rectangle {
        id: rectangle

        anchors.fill: parent
        color: actionButton.fillColor
        border.color: actionButton.borderColor
        border.width: units.dp(1)
        radius: units.dp(10)
        visible: actionButton.buttonGraphic == ""

        Rectangle {
            anchors.fill: parent
            color: "black"
            opacity: 0.2
            visible: mouseArea.pressed
        }
    }

    Rectangle {
        id: highlightRectangle

        anchors.fill: parent
        color: actionButton.fillColor
        border.color: actionButton.borderColor
        border.width: units.dp(1)
        radius: units.dp(10)
        visible: actionButton.buttonGraphic == ""

        Rectangle {
            anchors.fill: parent
            color: "black"
            opacity: 0.2
            visible: mouseArea.pressed
        }
    }

    TextDropShadow {
        target: textLabel
        opacity: 0.15
    }

    Text {
        id: textLabel
        text: actionButton.label
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        font.pixelSize: units.dp(16) // FIXME
        //font.family: notifications.font.name
        color: "white"
    }
}

