import QtQuick 2.0
import Widgets 0.1
import "settings.js" as Settings

Item {
    id: bubble
    
    property int topMargin: snapDecision ? units.dp(25/1.5) : units.dp(10)
    property int bottomMargin: snapDecision ? units.dp(25/1.5) : units.dp(10)
    property bool contracted: true
    property string icon
    property string title
    property string subtitle
    property variant actions
    property bool clickable: true
    property bool snapDecision: actions != null

    height: content.height + topMargin + bottomMargin
    //Behavior on height {NumberAnimation {duration: Settings.unityFastBeat; easing.type: Settings.easing }}

    signal clicked()
    signal selected()

    signal dragStarted()
    signal dismissed()
    signal restarted()

    signal focusChanged(bool focused)

    states: [
        State {
            name: "contracted"
            PropertyChanges { target: bubble; topMargin: units.dp(10); bottomMargin: units.dp(10) }
        },
        State {
            name: "expanded"
        }
    ]
    
    function restartFadeout() {
        fadeOutAnimation.restart()
    }

    BorderImageMipmap {
        id: snapDecisionBackground
        resource: "graphics/background.png"
        anchors.fill: parent
        border.left: units.dp(12); border.top: units.dp(12)
        border.right: units.dp(12); border.bottom: units.dp(12)
        opacity: 1
        visible: snapDecision
        smooth: true
    }

    BorderImageMipmap {
        id: clickableBackground
        resource: mouseArea.pressed ? "graphics/clickable-base-push.png" : "graphics/clickable-base.png"
        anchors.fill: parent
        border.left: units.dp(12); border.top: units.dp(12)
        border.right: units.dp(12); border.bottom: units.dp(12)
        opacity: 1
        visible: bubble.clickable && !snapDecision
        smooth: true
    }

    BorderImageMipmap {
        id: nonClickableBackground
        resource: "graphics/non-clickable-base.png"
        anchors.fill: parent
        border.left: units.dp(8/1.5); border.top: units.dp(8/1.5)
        border.right: units.dp(8/1.5); border.bottom: units.dp(8/1.5)
        visible: !bubble.clickable && !snapDecision
        opacity: 0.5
    }


    MouseArea {
        id: mouseArea
        property int initialY
        property bool toDismiss: initialY - bubble.y > units.dp(40/1.5)
        drag {
            target: bubble.snapDecision || !clickable ? null: bubble
            axis: Drag.YAxis
            minimumY: -bubble.height
            maximumY: 0
            onActiveChanged: mouseArea.drag.active ? bubble.dragStarted() : (mouseArea.toDismiss ? bubble.dismissed() : bubble.restarted())
        }
        anchors.fill: parent
        onPressed: initialY = bubble.y
        onClicked: bubble.clicked()
        onReleased: {
            if (mouseArea.toDismiss) {
                dismissAnimation.to = -1.5*bubble.height
            } else {
                dismissAnimation.to = 0
            }
            dismissAnimation.start()
        }
    }

    Content {
        id: content
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: parent.top
        anchors.topMargin: bubble.topMargin
        state: bubble.state
        actions: bubble.actions
        Behavior on anchors.topMargin {NumberAnimation { duration: Settings.unityFastBeat; easing.type: Settings.easing }}
    }

    NumberAnimation {
        id: dismissAnimation
        target: bubble
        property: "y"
        easing.type: Easing.OutQuad
        duration: 400
        to: -1.5*bubble.height
    }

    SequentialAnimation {
        id: fadeOutAnimation
        NumberAnimation {
            target: nonClickableBackground
            duration: 5000
            from: 1
            to: 0.5
            property: "opacity"
        }
    }
}
