import QtQuick 2.0
import Widgets 0.1
import "settings.js" as Settings

Item {
    id: content

    height: body.height
    property variant actions

    states: [
        State {
            name: "contracted"
            PropertyChanges { target: expandedContent; opacity: 0.0; height: 0; anchors.topMargin: 0 }
            StateChangeScript {
                script: splitButton.contract()
            }
        },
        State {
            name: "expanded"
            PropertyChanges { target: iconItem; width: units.dp(40/1.5); height: units.dp(40/1.5) }
        }
    ]

    Item {
        id: body

        anchors.left: content.left
        anchors.leftMargin: units.dp(19/1.5)
        anchors.right: content.right
        anchors.rightMargin: units.dp(19/1.5)
        height: childrenRect.height

        Item {
            id: iconFrame

            ImageMipmap {
                id: iconItem
                resource: bubble.icon // FIXME: reference to bubble
                smooth: true
                anchors.fill: parent

                Behavior on width {NumberAnimation { duration: Settings.unityFastBeat; easing.type: Settings.easing }}
                Behavior on height {NumberAnimation { duration: Settings.unityFastBeat; easing.type: Settings.easing }}
                fillMode: Image.PreserveAspectFit

                anchors.horizontalCenter: parent.horizontalCenter
            }

            width: height * iconItem.sourceSize.width / iconItem.sourceSize.height // preserve the original aspect ratio
            height: units.dp(56)
        }

        Item {
            id: contractedContent

            height: childrenRect.height
            anchors.left: iconFrame.right
            anchors.leftMargin: units.dp(25/1.5)
            anchors.right: parent.right

            Text {
                id: titleItem

                anchors.left: parent.left
                anchors.right: parent.right
                anchors.verticalCenter: bubble.subtitle == "" ? parent.verticalCenter : undefined // FIXME: reference to bubble
                elide: Text.ElideRight
                font.pixelSize: units.dp(34/1.5) // FIXME
                //font.family: notifications.font.name
                text: bubble.title // FIXME: reference to bubble
                color: bubble.clickable || bubble.snapDecision ? "white" : "#333333"
                //font.bold: true
                style: Text.Raised
                styleColor: Qt.rgba(0.0, 0.0, 0.0, 0.4)

                Rectangle {
                    anchors.fill: parent
                    color: "yellow"
                    opacity: 0
                }
            }

            Text {
                id: subtitleItem

                Rectangle {
                    anchors.fill: parent
                    color: "white"
                    opacity: 0
                }
                anchors.top: titleItem.bottom
                anchors.topMargin: units.dp(8/1.5)
                anchors.left: parent.left
                anchors.right: parent.right
                height: bubble.snapDecision ? lineCount*font.pixelSize : units.dp(10/1.5) + lineCount*font.pixelSize
                wrapMode: Text.Wrap
                maximumLineCount: 2
                elide: Text.ElideRight
                //font.family: notifications.font.name
                font.pixelSize: units.dp(20/1.5)// FIXME
                text: bubble.subtitle // FIXME: reference to bubble
                color: bubble.clickable || bubble.snapDecision ? "white" : "#333333"
                //font.bold: true
                style: Text.Raised
                styleColor: Qt.rgba(0.0, 0.0, 0.0, 0.4)
            }
        }

        Item {
            id: expandedContent

            height: childrenRect.height
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top: iconFrame.bottom
            anchors.topMargin: units.dp(22/1.5)

            Item {
                id: actions

                visible: content.actions != null
                //anchors.top: subtitleItem.bottom
                anchors.topMargin: units.dp(14/1.5)
                height: visible ? childrenRect.height + units.dp(2/1.5) : 0
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.rightMargin: units.dp(2)

                Row {
                    property int halfWidth: (actions.width - spacing) / 2
                    spacing: units.dp(20/1.5)
                    height: childrenRect.height
                    SplitButton {
                        id: splitButton
                        label: content.actions ? content.actions.action1 : ""
                        width: content.state == "expanded" ? actions.width : parent.halfWidth // FIXME
                        Behavior on width {NumberAnimation { duration: Settings.unitySlowBeat; easing.type: Settings.easing }}
                        onStateChanged: if (state == "expanded") bubble.state = "expanded" // FIXME: reference to bubble
                                        else bubble.state = ""
                        buttonHeight: units.dp(52/1.5)
                        expandedButtonHeight: units.dp(50)
                        onSelected: bubble.selected()
                        onMenuOpened: bubble.focusChanged(true)
                        onMenuClosed: bubble.focusChanged(false)
                    }

                    ActionButton {
                        id: acceptButton
                        buttonGraphic: "graphics/btn_Accept.png"
                        pressedGraphic: "graphics/btn_Accept_Push.png"
                        label: content.actions ? content.actions.action2 : ""
                        width: parent.halfWidth // FIXME
                        opacity: content.state == "expanded" ? 0.0 : 1.0
                        height: units.dp(52/1.5)
                        touchAreaMargins: -units.dp(10/1.5)

                        Behavior on opacity {NumberAnimation { duration: Settings.unitySlowBeat; easing.type: Settings.easing }}

                        onSelected: bubble.selected()
                    }
                }
            }
        }
    }
}
