import QtQuick 2.0
import Widgets 0.1
import "settings.js" as Settings

Item {
    id: splitButton

    property string label
    property real fillAlpha: 0.1
    property color fillColor: Qt.rgba(1.0, 1.0, 1.0, fillAlpha)
    property color borderColor: Qt.rgba(1.0, 1.0, 1.0, 0.25)

    // Mouse area margins
    property int progressiveGestureTopHeight: units.dp(125/1.5)
    property int buttonLeftMargin: units.dp(10)
    property int buttonRightMargin: units.dp(8)
    property int buttonTopMargin: units.dp(20/1.5)
    property int buttonBottomMargin: units.dp(20/1.5)

    property int __chevronRightMargin: units.dp(10)
    property bool __progressiveSelection: false

    signal selected()
    signal menuOpened()
    signal menuClosed()

    onStateChanged: state == "" ? splitButton.menuClosed() : splitButton.menuOpened()
    property int buttonHeight: units.dp(50/1.5)
    property int expandedButtonHeight: units.dp(50)

    height: childrenRect.height

    function contract() {
        splitButton.state = ""
    }

    function __selectCurrent() {
        // here we could check the highlight index value to get the selected index
        if (highlight.index >= 0) {
            splitButton.selected()
        }
    }

    BorderImageMipmap {
        id: image
        anchors.fill: all
        border { left: units.dp(10/1.5); top: units.dp(10/1.5); right: units.dp(10/1.5); bottom: units.dp(10/1.5) }
        horizontalTileMode: BorderImage.Stretch
        verticalTileMode: BorderImage.Stretch
        resource: "graphics/btn_Decline.png"
    }

    Item {
        id: highlight

        property int index: 0
        property color color: "#dddddd"
        anchors.left: parent.left
        anchors.right: parent.right
        y: index * height
        height: index >= 0 ? (splitButton.state == "" ? buttonHeight: expandedButtonHeight) : 0
        visible: splitButton.__progressiveSelection && index >= 0 && !dragMouse.chevronPress

        Rectangle {
            anchors.fill: parent
            anchors.margins: units.dp(4/1.5)
            anchors.bottomMargin: units.dp(2)
            color: highlight.color
            radius: units.dp(10)
        }

        Rectangle {
            anchors.fill: parent
            anchors.margins: units.dp(4/1.5)
            anchors.topMargin: units.dp(2/1.5)
            anchors.bottomMargin: highlight.height / 2
            width: highlight.width
            color: highlight.color
            visible: highlight.index != 0
        }

        Rectangle {
            anchors.fill: parent
            anchors.margins: units.dp(4/1.5)
            anchors.topMargin: highlight.height / 2
            anchors.bottomMargin: units.dp(2/1.5)
            color: highlight.color
            visible: highlight.index != 3 && splitButton.state == "expanded"
        }

    }

    MouseArea {
        anchors.fill: all
        onClicked: {
            splitButton.selected()
        }
    }

    Item {
        id: all

        height: childrenRect.height
        anchors.left: parent.left
        anchors.right: parent.right

        function calculateHighlight(x, y, addOffset) {
            var originalY = y
            var offset = 2 * splitButton.expandedButtonHeight / 3
            if (addOffset) y -= offset

            if (childAt(x, y) == mainOption || childAt(x, originalY) == mainOption) {
                highlight.index = 0
            } else if (x > mainOption.x
                       && x < mainOption.x + mainOption.width + buttonRightMargin
                       && originalY > mainOption.y - buttonTopMargin
                       && originalY < mainOption.y + mainOption.height + buttonBottomMargin) {
                highlight.index = 0
            } else if (childAt(x,  y) == extraOptions) {
                var mapped = extraOptions.mapFromItem(all, x, y)
                var childItem = extraOptions.childAt(mapped.x, mapped.y)
                if (childItem) {
                    highlight.index = childItem.ownIndex + 1
                }
            } else {
                highlight.index = -1
            }
        }

        Item {
            id: mainOption

            height: buttonHeight
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.leftMargin: units.dp(2/1.5)
            anchors.rightMargin: units.dp(2/1.5)

            MouseArea {
                id: dragMouse

                property int initialY
                property bool chevronPress: false
                property bool chevronRelease: false
                property bool chevronClick: chevronPress && chevronRelease

                anchors.fill: parent
                anchors.topMargin: -Math.max(progressiveGestureTopHeight, buttonTopMargin)
                anchors.bottomMargin: -Math.max(extraOptions.height, buttonBottomMargin)
                anchors.leftMargin: -buttonLeftMargin
                anchors.rightMargin: -buttonRightMargin

                onPressed: {
                    var mappedToAll = mapToItem(all, mouseX, mouseY)
                    var mappedToChevron = mapToItem(chevron, mouseX, mouseY)
                    initialY = mouseY
                    all.calculateHighlight(mappedToAll.x, mappedToAll.y, false)
                    splitButton.__progressiveSelection = true

                    // check if chevron was pressed
                    chevronPress = chevron.contains(mappedToChevron.x, mappedToChevron.y)
                }
                onMouseYChanged: {
                    var mappedToAll = mapToItem(all, mouseX, mouseY)
                    if (mappedToAll.y > mainOption.height/2 && (mouseY - initialY > units.dp(10/1.5))) {
                        splitButton.state = "expanded"
                        chevronPress = false
                    }
                    if (mouseY - initialY > units.dp(10/1.5) || mouseY - initialY < -units.dp(10/1.5)) {
                        all.calculateHighlight(mappedToAll.x, mappedToAll.y, true)
                    }
                }
                onReleased: {
                    // check if chevron was pressed
                    var mappedToChevron = mapToItem(chevron, mouseX, mouseY)
                    chevronRelease = chevron.contains(mappedToChevron.x, mappedToChevron.y)

                    if (splitButton.state == "expanded" && !chevronClick) {
                        splitButton.__selectCurrent()
                    } else if (splitButton.state == "" && chevronClick) {
                        splitButton.state = "expanded"
                    } else if (splitButton.state == "expanded" && chevronClick) {
                        splitButton.state = ""
                    }
                    splitButton.__progressiveSelection = false
                }
                onClicked: {
                    if (!chevronClick) {
                        splitButton.__selectCurrent()
                    }
                }
            }

            Rectangle {
                id: chevron

                anchors.top: parent.top
                anchors.bottom: parent.bottom
                anchors.topMargin: units.dp(2/1.5)
                anchors.bottomMargin: units.dp(2/1.5)
                width: height + units.dp(2/1.5)
                color: "transparent"
                radius: units.dp(10)

                function contains(mappedX, mappedY) {
                    if ( mappedX > -buttonLeftMargin
                        && mappedX < chevron.width + __chevronRightMargin
                        && mappedY > -buttonTopMargin
                        && mappedY < chevron.height + buttonBottomMargin ) {
                        return true
                    } else {
                        return false
                    }
                }
                ImageMipmap {
                    id: lineGraphic
                    anchors {
                        right: parent.right
                        rightMargin: -units.dp(2/1.5)
                        top: parent.top
                        topMargin: units.dp(2/1.5)
                        bottom: parent.bottom
                    }
                    resource: "graphics/div.png"
                }

                ImageMipmap {
                    id: arrow
                    anchors.centerIn: parent
                    fillMode: Image.PreserveAspectFit
                    resource: "graphics/arrow.png"
                    rotation: 0
                }
            }

            Item {
                id: mainOptionLabel
                anchors{
                    left: parent.left
                    right: parent.right
                    top: parent.top
                    bottom: parent.bottom
                }

                TextDropShadow {
                    target: text
                }
                Text {
                    id: text

                    property real alfa: (mainOption.height - splitButton.expandedButtonHeight) / (splitButton.buttonHeight - splitButton.expandedButtonHeight)
                    text: splitButton.label
                    x: (splitButton.width - paintedWidth) / 2 + alfa * chevron.width / 4
                    anchors.verticalCenter: parent.verticalCenter
                    font.pixelSize: units.dp(16) // FIXME
                    //font.family: notifications.font.name
                    color: "#333333"
                }
            }
        }

        Column {
            id: extraOptions

            anchors.top: mainOption.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            height: 0
            opacity: 0.0
            clip: true

            Repeater {
                model: optionsModel
                delegate: Item {
                    anchors.left: parent.left
                    anchors.right: parent.right
                    height: childrenRect.height // FIXME: bottom margin units.dp(10/1.5)
                    property int ownIndex: index
                    Item {
                        anchors.left: parent.left
                        anchors.right: parent.right
                        height: expandedButtonHeight

                        ImageMipmap {
                            anchors {
                                left: parent.left
                                right: parent.right
                                leftMargin: units.dp(4/1.5)
                                rightMargin: units.dp(4/1.5)
                            }
                            resource: "graphics/div-horizontal.png"
                        }

                        TextDropShadow {
                            target: text
                        }

                        Text {
                            id: text
                            anchors.left: parent.left
                            anchors.right: parent.right
                            anchors.verticalCenter: parent.verticalCenter
                            text: label
                            y: units.dp(10/1.5) // FIXME: top margin 10
                            horizontalAlignment: Text.AlignHCenter
                            font.pixelSize: units.dp(16) // FIXME
                            //font.family: notifications.font.name
                            color: "#333333"
                        }
                    }
                }
            }
        }
    }

    ListModel {
        id: optionsModel

        ListElement {
            label: "\"Can't talk now, what's up?\""
        }
        ListElement {
            label: "\"I'll call you back later.\""
        }
        ListElement {
            label: "Send other message..."
        }
    }

    states: [
        State {
            name: "expanded"
            PropertyChanges { target: arrow; rotation: 90 }
            PropertyChanges { target: mainOption; height: expandedButtonHeight }
            PropertyChanges { target: mainOptionLabel; width: splitButton.width - chevron.width * 2 }
            PropertyChanges { target: extraOptions; height: childrenRect.height; opacity: 1.0 }
        }
    ]

    transitions: [
        Transition {
            from: "expanded"
            to: ""
            NumberAnimation {
                target: mainOption
                property: "height"
                duration: Settings.unitySlowBeat;
                easing.type: Settings.easing
            }
            NumberAnimation {
                target: arrow
                property: "rotation"
                duration: Settings.unitySlowBeat;
                easing.type: Settings.easing
            }
            NumberAnimation {
                target: extraOptions
                duration: Settings.unitySlowBeat;
                easing.type: Settings.easing
                properties: "height,opacity"
            }
        },
        Transition {
            from: ""
            to: "expanded"
            NumberAnimation {
                target: mainOption
                property: "height"
                duration: Settings.unitySlowBeat;
                easing.type: Settings.easing
            }
            NumberAnimation {
                target: arrow
                property: "rotation"
                duration: Settings.unitySlowBeat;
                easing.type: Settings.easing
            }
            NumberAnimation {
                target: extraOptions
                duration: Settings.unitySlowBeat;
                easing.type: Settings.easing
                properties: "height,opacity"
            }
        }
    ]
}
