/*
 * Copyright 2012 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0

BorderImage {
    id: image

    property url resource
    property real scaleFactor: fallback.chosenSourceIndex == -1 ? 1 : units.scaleFactor / __chosenAssetFactor


    ImageWithFallback {
        id: fallback
        image: image
        fallbackSources: [__appendSuffix(resource, units.categories[2][0]),
                          __appendSuffix(resource, units.categories[1][0]),
                          __appendSuffix(resource, units.categories[0][0]),
                         ]
    }

    property real __chosenAssetFactor: fallback.chosenSourceIndex == -1 ? 0.0 : units.categories[__chosenAssetCategory][2]
    property int __chosenAssetCategory: units.categories.length - 1 - fallback.chosenSourceIndex
    property bool __needsDownscaling: true
    property bool __needsUpscaling: true

    onScaleFactorChanged: {downscale(); upscale()}
    onStatusChanged: {downscale(); upscale()}

    function upscale() {
        if (__needsUpscaling && status == Image.Ready && scaleFactor > 1
            && width == implicitWidth && height == implicitHeight ) {
            __needsUpscaling = false
            width = implicitWidth * scaleFactor
            height = implicitHeight * scaleFactor
        }
    }

    function downscale() {
        if (__needsDownscaling && status == Image.Ready && scaleFactor < 1) {
            __needsDownscaling = false
            sourceSize.width = implicitWidth * scaleFactor
            sourceSize.height = implicitHeight * scaleFactor
        }
    }

    source: __appendSuffix(resource, units.suffix)

    function __appendSuffix(resourceUrl, suffix) {
        if (resourceUrl == "") return ""
        var resourceString = resourceUrl.toString()
        var baseName = resourceString.substring(0, resourceString.length - 4)
        var extension = resourceString.substring(resourceString.length - 4, resourceString.length)
        return baseName + suffix + extension
    }

    smooth: true
}
